--This populates an LOD list based on the appropriate LOD names and then allows you to switch them quickly and see
--polygon counts.
Rollout FSLODTool "Switch LODs!"
(
	button populate "Populate the LODs!" width:300
	editText pCountVal "Polygon Count:" fieldWidth:50 text:""
	multiListbox allLOD items:#() height:8
	
	--Set up the function to repopulate the LOD list based on the object's LODs.
	fn populateLODList =
	(
		local polyString = "0"
		local allObj = #()
		for obj in $*_LOD_* do
		(
			if findItem AllObj obj.name == 0 then
			(
				append AllObj obj.name
			)
		)
		append AllObj "Unhide All"
		return allObj
	)
	
	--When the rollout is open, populate the LOD list
	on FSLODTool open do
	(
		allLOD.items = populateLODList ()
	)
	
	--Define a function to gather the LOD numbers from the objects
	fn fnDefineLODNum obj =
	(
		objNameString = obj.name as stringstream
		local thisPos = findString (objNameString as string) "_LOD_"
		thisPos += 4
		local LODNumSize = (objNameString as string).count - thisPos
		local thisNum = (subString (objNameString as string) (thisPos + 1) LODNumSize)
		return thisNum 
	)
	
	on populate pressed do
	(
		allLOD.selection = #{}
		allLOD.items = populateLODList ()
	)
	on allLOD selected LODName do
	(
		if allLOD.items[LODName] == "Unhide All" then
		(
			allLOD.selection = #{}
			max unhide all
			clearSelection ()
			pCountVal.text = "0"
		)
		else
		(
			try
			(
				hide $*
				global dummyString = 0
				for obj in allLOD.selection do
				(
					singleObj = execute ("$" + allLOD.items[obj])
					newObjName = execute ("$" + singleObj.name + "...*")
					unhide newObjName
					select newObjName
					for objChildren in $ do
					(
						try
						(
							dummyString += objChildren.faces.count
						)
						catch ()
					)
				)
				pCountVal.text = dummyString as string
			)
			catch()
		)
	)
	
	--An easy way to update the names of a file.  Rename the Max file name, then press this button.  It will rename the
	--LODs for you.
	button renBuild "Rename the LODs to match Filename!" width:300
	on renBuild pressed do
	(
		local LODname = getfilenamefile maxFileName
		for obj in $*_LOD_* do
		(
			LODNumber = fnDefineLODNum obj
			obj.name = (LODName + "_LOD_" + LODNumber)
		)
		if rootnode.children.count == 1 then rootNode.children.name = LODName
		allLOD.items = populateLODList ()
	)
)

--Create a rollout for getting and creating new GUID and friendly names.
Rollout getSetFSGUID "Get and Set the GUID and Friendly Name"
(
	checkBox boolChooseXFile "Choose the .X File to get the GUID from?" checked:false
	button getMaxGUIDX "Get GUID from .x file" across:2 align:#left width:145 pos:[10,30]
	button createGUIDX "Create new GUID for this file" align:#left width:145 pos:[165,30]
	
	--Setup a function that allows you to add a new GUID or Friendly name, but queries the user to replace if one already exists
	local strGUIDOrFriendly, strNewGUIDOrFriendly
	fn queryGUIDAndFriendly strGUIDOrFriendly strNewGUIDOrFriendly = 
	(
		if (fileProperties.findProperty #custom strGUIDOrFriendly) == 0 then
		(
			fileProperties.addProperty #custom strGUIDOrFriendly strNewGUIDOrFriendly
		)
		else
		(
			if (queryBox ("This file already has a " + strGUIDOrFriendly + " associated with it.  Should I replace it?")) then
			(
				fileProperties.deleteProperty #custom strGUIDOrFriendly
				fileProperties.addProperty #custom strGUIDOrFriendly strNewGUIDOrFriendly
			)
		)
	)
	
	--Get GUID from .x file to make sure we aren't blowing away the real GUID.
	on getMaxGUIDX pressed do
	(
		try
		(
			local strThisFileX
			if boolChooseXFile.checked == true then
			(
				strThisFileX = openFile (getOpenFileName caption:"Choose .x file to grab the GUID and friendly from." filename:(maxFilePath + "*.x"))
			)
			else
			(
				strThisFileX = openFile (maxFilePath + (getFilenameFile maxFilename) + ".x")
			)
			skipToString strThisFileX "GuidToName {"
			skipToString strThisFileX "GuidToName {"
			if (skipToString strThisFileX "GuidToName {") != undefined then
			(
				strThisFileGUIDFromX = "badGUID"
				strThisFileFriendlyFromX = "badFriendly"
			)
			else
			(
				seek strThisFileX 0
				skipToString strThisFileX "GuidToName {"
				skipToString strThisFileX "GuidToName {"
				skipToString strThisFileX "{"
				strThisFileGUIDFromX = readDelimitedString strThisFileX "}"
				skipToNextLine strThisFileX
				skipToString strThisFileX "\""
				strThisFileFriendlyFromX = readDelimitedString strThisFileX "\""
			)
			
			--Use query function to test for existing GUID or Friendly Name
			if strThisFileGUIDFromX == "badGUID" then
			(
				messageBox "The GUID in this .X file is screwed up.  I'm not going to grab it."
			)
			else
			(
				queryGUIDAndFriendly "Guid" strThisFileGUIDFromX
			)
			if strThisFileFriendlyFromX == "badFriendly" then
			(
				messageBox "The Friendly Name in this .X file is screwed up.  I'm not going to grab it."
			)
			else
			(
				queryGUIDAndFriendly "FriendlyName" strThisFileFriendlyFromX
			)
			
			--Close the X file so it can be overwritten
			close strThisFileX
		)
		catch (messageBox "Either you didn't select an .x file, or I can't find it.  Please try again.")
	)
	
	--Create a new GUID and Firendly Name for this file
	global friendlyNameRollout
	on createGUIDX pressed do
	(
		--Setup rollout for the Friendly Name creation
		Rollout friendlyNameRollout "Friendly Name info"
		(
			editText friendlyNameTextBox "Friendly Name:" text:(getFilenameFile maxFilename)
			button goFriendlyName "Go!"
			on goFriendlyName pressed do
			(
				queryGUIDAndFriendly "FriendlyName" friendlyNameTextBox.text
				destroyDialog friendlyNameRollout
			)
		)
		local newGUIDX = (maxUtils.getGUID()) as stringStream
		skipToString newGUIDX "{"
		local thisFSGUID = readDelimitedString newGUIDX "}"
		queryGUIDAndFriendly "Guid" (thisFSGUID as string)
		try (destroyDialog friendlyNameRollout) catch ()
		createDialog friendlyNameRollout 300 60
	)
)

--Create the rollout floater with both tools
FSLODAndNameFloater = newRolloutFloater "LOD and GUID tools" 330 350
addRollout FSLODTool FSLODAndNameFloater
addRollout getSetFSGUID FSLODAndNameFloater